@@ -10,7 +10,10 @@ import timeit
 import logging
 import warnings
 from datetime import datetime, timedelta
-import collections
+try:  # pragma: no cover
+    from collections import abc
+except ImportError:  # pragma: no cover
+    import collections as abc
 
 from python_utils import converters
 
@@ -47,7 +50,7 @@ class ProgressBarMixinBase(object):
                 pass
 
 
-class ProgressBarBase(collections.Iterable, ProgressBarMixinBase):
+class ProgressBarBase(abc.Iterable, ProgressBarMixinBase):
     pass
 
 
@@ -227,9 +230,9 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
     _DEFAULT_MAXVAL = base.UnknownLength
     _MINIMUM_UPDATE_INTERVAL = 0.05  # update up to a 20 times per second
 
-    def __init__(self, min_value=0, max_value=None,
-                 widgets=None, left_justify=True, initial_value=0,
-                 poll_interval=None, widget_kwargs=None, custom_len=len,
+    def __init__(self, min_value=0, max_value=None, widgets=None,
+                 left_justify=True, initial_value=0, poll_interval=None,
+                 widget_kwargs=None, custom_len=utils.len_color,
                  max_error=True, prefix=None, suffix=None, **kwargs):
         '''
         Initializes a progress bar with sane defaults
@@ -434,14 +437,13 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
 
     def __call__(self, iterable, max_value=None):
         'Use a ProgressBar to iterate through an iterable'
-        if max_value is None:
+        if max_value is not None:
+            self.max_value = max_value
+        elif self.max_value is None:
             try:
                 self.max_value = len(iterable)
             except TypeError:  # pragma: no cover
-                if self.max_value is None:
-                    self.max_value = base.UnknownLength
-        else:
-            self.max_value = max_value
+                self.max_value = base.UnknownLength
 
         self._iterable = iter(iterable)
         return self
@@ -529,6 +531,7 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
             delta = timeit.default_timer() - self._last_update_timer
             poll_status = delta > self.poll_interval.total_seconds()
         else:
+            delta = 0
             poll_status = False
 
         # Do not update if value increment is not large enough to
@@ -538,11 +541,13 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
             divisor = self.max_value / self.term_width  # float division
             if self.value // divisor == self.previous_value // divisor:
                 return poll_status or self.end_time
+            else:
+                return True
         except Exception:
             # ignore any division errors
             pass
 
-        return self.value > self.next_update or poll_status or self.end_time
+        return poll_status or self.end_time
 
     def update(self, value=None, force=False, **kwargs):
         'Updates the ProgressBar to a new value.'
@@ -554,7 +559,7 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
             if self.max_value is base.UnknownLength:
                 # Can't compare against unknown lengths so just update
                 pass
-            elif self.min_value <= value <= self.max_value:
+            elif self.min_value <= value <= self.max_value:  # pragma: no cover
                 # Correct value, let's accept
                 pass
             elif self.max_error:
@@ -647,7 +652,6 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
                 )
 
         self.num_intervals = max(100, self.term_width)
-        self.next_update = 0
 
         if self.max_value is not base.UnknownLength and self.max_value < 0:
             raise ValueError('Value out of range')
@@ -658,7 +662,7 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
 
         return self
 
-    def finish(self, end='\n'):
+    def finish(self, end='\n', dirty=False):
         '''
         Puts the ProgressBar bar in the finished state.
 
@@ -668,10 +672,13 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
         Args:
             end (str): The string to end the progressbar with, defaults to a
                 newline
+            dirty (bool): When True the progressbar kept the current state and
+                won't be set to 100 percent
         '''
 
-        self.end_time = datetime.now()
-        self.update(self.max_value, force=True)
+        if not dirty:
+            self.end_time = datetime.now()
+            self.update(self.max_value, force=True)
 
         StdRedirectMixin.finish(self, end=end)
         ResizableMixin.finish(self)
