@@ -65,13 +65,17 @@ class FormatWidgetMixin(object):
     '''
     required_values = []
 
-    def __init__(self, format, **kwargs):
+    def __init__(self, format, new_style=False, **kwargs):
+        self.new_style = new_style
         self.format = format
 
     def __call__(self, progress, data, format=None):
         '''Formats the widget into a string'''
         try:
-            return (format or self.format) % data
+            if self.new_style:
+                return (format or self.format).format(**data)
+            else:
+                return (format or self.format) % data
         except (TypeError, KeyError):
             print('Error while formatting %r' % self.format, file=sys.stderr)
             pprint.pprint(data, stream=sys.stderr)
@@ -173,6 +177,10 @@ class FormatLabel(FormatWidgetMixin, WidthWidgetMixin):
     >>> str(label(Progress, dict(value='test')))
     ''
 
+    >>> label = FormatLabel('{value} :: {value:^6}', new_style=True)
+    >>> str(label(Progress, dict(value='test')))
+    'test ::  test '
+
     '''
 
     mapping = {
@@ -232,19 +240,20 @@ class SamplesMixin(TimeSensitiveWidgetBase):
     >>> samples(progress, None, True)
     (None, None)
     >>> progress.last_update_time += datetime.timedelta(seconds=1)
-    >>> samples(progress, None, True)
-    (datetime.timedelta(0, 1), 0)
+    >>> samples(progress, None, True) == (datetime.timedelta(seconds=1), 0)
+    True
+
     >>> progress.last_update_time += datetime.timedelta(seconds=1)
-    >>> samples(progress, None, True)
-    (datetime.timedelta(0, 1), 0)
+    >>> samples(progress, None, True) == (datetime.timedelta(seconds=1), 0)
+    True
 
     >>> samples = SamplesMixin(samples=datetime.timedelta(seconds=1))
     >>> _, value = samples(progress, None)
     >>> value
     [1, 1]
 
-    >>> samples(progress, None, True)
-    (datetime.timedelta(0, 1), 0)
+    >>> samples(progress, None, True) == (datetime.timedelta(seconds=1), 0)
+    True
     '''
 
     def __init__(self, samples=datetime.timedelta(seconds=2), key_prefix=None,
@@ -273,8 +282,11 @@ class SamplesMixin(TimeSensitiveWidgetBase):
             sample_values.append(progress.value)
 
             if isinstance(self.samples, datetime.timedelta):
-                begin = progress.last_update_time - self.samples
-                while sample_times[2:] and begin > sample_times[0]:
+                minimum_time = progress.last_update_time - self.samples
+                minimum_value = sample_values[-1]
+                while (sample_times[2:] and
+                       minimum_time > sample_times[1] and
+                       minimum_value > sample_values[1]):
                     sample_times.pop(0)
                     sample_values.pop(0)
             else:
@@ -299,9 +311,9 @@ class ETA(Timer):
     def __init__(
             self,
             format_not_started='ETA:  --:--:--',
-            format_finished='Time: %(elapsed)s',
-            format='ETA:  %(eta)s',
-            format_zero='ETA:  0:00:00',
+            format_finished='Time: %(elapsed)8s',
+            format='ETA:  %(eta)8s',
+            format_zero='ETA:  00:00:00',
             format_NA='ETA:      N/A',
             **kwargs):
 
@@ -316,7 +328,7 @@ class ETA(Timer):
         '''Updates the widget to show the ETA or total time when finished.'''
         if elapsed:
             # The max() prevents zero division errors
-            per_item = elapsed / max(value, 0.000000001)
+            per_item = elapsed.total_seconds() / max(value, 1e-6)
             remaining = progress.max_value - data['value']
             eta_seconds = remaining * per_item
         else:
@@ -331,7 +343,7 @@ class ETA(Timer):
             value = data['value']
 
         if elapsed is None:
-            elapsed = data['total_seconds_elapsed']
+            elapsed = data['time_elapsed']
 
         ETA_NA = False
         try:
@@ -341,10 +353,12 @@ class ETA(Timer):
             data['eta_seconds'] = None
             ETA_NA = True
 
+        data['eta'] = None
         if data['eta_seconds']:
-            data['eta'] = utils.format_time(data['eta_seconds'])
-        else:
-            data['eta'] = None
+            try:
+                data['eta'] = utils.format_time(data['eta_seconds'])
+            except (ValueError, OverflowError):  # pragma: no cover
+                pass
 
         if data['value'] == progress.min_value:
             format = self.format_not_started
@@ -377,10 +391,8 @@ class AbsoluteETA(ETA):
             format_finished='Finished at: %(elapsed)s',
             format='Estimated finish time: %(eta)s',
             **kwargs):
-        Timer.__init__(self, **kwargs)
-        self.format_not_started = format_not_started
-        self.format_finished = format_finished
-        self.format = format
+        ETA.__init__(self, format_not_started=format_not_started,
+                     format_finished=format_finished, format=format, **kwargs)
 
 
 class AdaptiveETA(ETA, SamplesMixin):
@@ -397,7 +409,6 @@ class AdaptiveETA(ETA, SamplesMixin):
     def __call__(self, progress, data):
         elapsed, value = SamplesMixin.__call__(self, progress, data,
                                                delta=True)
-
         if not elapsed:
             value = None
             elapsed = 0
@@ -697,15 +708,19 @@ class FormatCustomText(FormatWidgetMixin, WidthWidgetMixin):
         self.mapping.update(mapping)
 
     def __call__(self, progress, data):
-        return FormatWidgetMixin.__call__(self, progress, self.mapping,
-                                          self.format)
+        return FormatWidgetMixin.__call__(
+            self, progress, self.mapping, self.format)
 
 
 class DynamicMessage(FormatWidgetMixin, WidgetBase):
     '''Displays a custom variable.'''
 
-    def __init__(self, name):
+    def __init__(self, name, format='{name}: {formatted_value}',
+                 width=6, precision=3):
         '''Creates a DynamicMessage associated with the given name.'''
+        self.format = format
+        self.width = width
+        self.precision = precision
         if not isinstance(name, str):
             raise TypeError('DynamicMessage(): argument must be a string')
         if len(name.split()) > 1:
@@ -715,11 +730,24 @@ class DynamicMessage(FormatWidgetMixin, WidgetBase):
         self.name = name
 
     def __call__(self, progress, data):
-        val = data['dynamic_messages'][self.name]
-        if val:
-            return self.name + ': ' + '{:6.3g}'.format(val)
-        else:
-            return self.name + ': ' + 6 * '-'
+        value = data['dynamic_messages'][self.name]
+        context = data.copy()
+        context['value'] = value
+        context['name'] = self.name
+        context['width'] = self.width
+        context['precision'] = self.precision
+
+        try:
+            context['formatted_value'] = '{value:{width}.{precision}}'.format(
+                **context)
+        except (TypeError, ValueError):
+            if value:
+                context['formatted_value'] = '{value:{width}}'.format(
+                    **context)
+            else:
+                context['formatted_value'] = '-' * self.width
+
+        return self.format.format(**context)
 
 
 class CurrentTime(FormatWidgetMixin, TimeSensitiveWidgetBase):
